@@ -42,6 +42,7 @@ import getpass
 import email
 import poplib
 import imaplib
+import re
 
 try:
     # do we have a recent pykerberos?
@@ -80,6 +81,28 @@ VANISHED_AGE = (60 * 60 * 24 * 30)
 # Kerberos authentication state constants
 (GSS_STATE_STEP, GSS_STATE_WRAP) = (0, 1)
 
+#
+# Bugfix classes
+#
+
+if sys.version_info < (2, 5, 3):
+    # A serious imaplib bug (http://bugs.python.org/issue1389051) was
+    # fixed in 2.5.3.  Earlier Python releases need a work-around.
+    orig_imap4_ssl = imaplib.IMAP4_SSL
+    class IMAP4_SSL(orig_imap4_ssl):
+        def read(self, size):
+            """Read 'size' bytes from remote."""
+            # sslobj.read() sometimes returns < size bytes
+            chunks = []
+            read = 0
+            while read < size:
+                data = self.sslobj.read(min(size-read, 16384))
+                read += len(data)
+                chunks.append(data)
+            return ''.join(chunks)
+    
+    imaplib.IMAP4_SSL = IMAP4_SSL
+
 #
 # Mix-in classes
 #
@@ -92,6 +115,7 @@ class POP3initMixIn(object):
         self.log.trace()
         try:
             self.conn = poplib.POP3(self.conf['server'], self.conf['port'])
+            self.setup_received(self.conn.sock)
         except poplib.error_proto, o:
             raise getmailOperationError('POP error (%s)' % o)
         except socket.timeout:
@@ -135,6 +159,7 @@ class Py24POP3SSLinitMixIn(object):
                                + os.linesep)
                 self.conn = poplib.POP3_SSL(self.conf['server'],
                                             self.conf['port'])
+            self.setup_received(self.conn.sock)
         except poplib.error_proto, o:
             raise getmailOperationError('POP error (%s)' % o)
         except socket.timeout:
@@ -179,6 +204,7 @@ class Py23POP3SSLinitMixIn(object):
                     + os.linesep
                 )
                 self.conn = POP3SSL(self.conf['server'], self.conf['port'])
+            self.setup_received(self.conn.rawsock)
         except poplib.error_proto, o:
             raise getmailOperationError('POP error (%s)' % o)
         except socket.timeout:
@@ -201,6 +227,7 @@ class IMAPinitMixIn(object):
         self.log.trace()
         try:
             self.conn = imaplib.IMAP4(self.conf['server'], self.conf['port'])
+            self.setup_received(self.conn.sock)
         except imaplib.IMAP4.error, o:
             raise getmailOperationError('IMAP error (%s)' % o)
         except socket.timeout:
@@ -241,6 +268,7 @@ class IMAPSSLinitMixIn(object):
                 )
                 self.conn = imaplib.IMAP4_SSL(self.conf['server'],
                                               self.conf['port'])
+            self.setup_received(self.conn.sock)
         except imaplib.IMAP4.error, o:
             raise getmailOperationError('IMAP error (%s)' % o)
         except socket.timeout:
@@ -328,6 +356,21 @@ class RetrieverSkeleton(ConfigurableBase):
         self.gotmsglist = False
         ConfigurableBase.__init__(self, **args)
 
+    def setup_received(self, sock):
+        serveraddr = sock.getpeername()
+        if len(serveraddr) == 2:
+            # IPv4
+            self.remoteaddr = '%s:%s' % serveraddr
+        elif len(serveraddr) == 4:
+            # IPv6
+            self.remoteaddr = '[%s]:%s' % serveraddr[:2]
+        else:
+            # Shouldn't happen
+            self.log.warn('unexpected peer address format %s', str(serveraddr))
+            self.remoteaddr = str(serveraddr)
+        self.received_from = '%s (%s)' % (self.conf['server'], 
+                                          self.remoteaddr)
+
     def __del__(self):
         self.log.trace()
         self.write_oldmailfile()
@@ -410,16 +453,16 @@ class RetrieverSkeleton(ConfigurableBase):
         else:
             # Explicitly set to None in case it was previously set
             socket.setdefaulttimeout(None)
-        self.oldmail_filename = os.path.join(
-            self.conf['getmaildir'],
-            ('oldmail-%(server)s-%(port)i-%(username)s'
-             % self.conf).replace('/', '-').replace(':', '-')
+        # strip problematic characters from oldmail filename.  Mostly for
+        # non-Unix systems; only / is illegal in a Unix path component
+        oldmail_filename = re.sub(
+            r'[/\:;<>|]+', '-',
+            'oldmail-%(server)s-%(port)i-%(username)s' % self.conf
         )
+        self.oldmail_filename = os.path.join(self.conf['getmaildir'], 
+                                             oldmail_filename)
         self._read_oldmailfile()
-        self.received_from = '%s (%s)' % (
-            self.conf['server'],
-            socket.getaddrinfo(self.conf['server'], None)[0][4][0]
-        )
+        self.received_from = None
         self.__initialized = True
 
     def delivered(self, msgid):
@@ -785,7 +828,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         self.log.debug('selecting mailbox "%s"' % mailbox + os.linesep)
         try:
             response = self._parse_imapcmdresponse('SELECT', mailbox)
-            count = int(response[0])
+            count = int(response[-1]) # use *last* EXISTS returned
             uidvalidity = self.conn.response('UIDVALIDITY')[1][0]
         except imaplib.IMAP4.error, o:
             raise getmailOperationError('IMAP error (%s)' % o)
@@ -882,7 +925,14 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             # (virus, spam, trojan), it can completely fail to return the
             # message when requested.
             try:
-                msg = Message(fromstring=response[0][1])
+                try:
+                    sbody = response[0][1]
+                except Exception, o:
+                    sbody = None
+                if not sbody:
+                    self.log.error('bad message from server!')
+                    sbody = str(response)
+                msg = Message(fromstring=sbody)
             except TypeError, o:
                 # response[0] is None instead of a message tuple
                 raise getmailRetrievalError('failed to retrieve msgid %s' 
